-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

---------------------
-- Prep the module --
---------------------
local M = {}
M.type = "auxilliary"
M.relevantDevice = nil
M.defaultOrder = 11000

-----------------
-- Debug stuff --
-----------------
local ugfxCalled = 0

---------------------------
-- Tuning menu variables --
---------------------------

-- Air brake momentary/toggle
local airBrakeToggle = 0

-- Hover momentary/toggle
local hoverToggle = 1

---------------------
-- State variables --
---------------------

-- Last trigger state of air brake and hover
local lastTrigAirBrake = 0
local lastTrigHover = 0
local lastParkBrake = 0



------------------------------------
-- Determine edges on triggers    --
------------------------------------

-- Edge events
-- 0 = No edge
-- 1 = Rising edge
-- 2 = Falling edge

local function getTriggerEdges()
	local	edgeDirAB = 0
	local	edgeDirH = 0
	local	edgeDirPB = 0
	
	-- Trigger activated
	local airBrakeTrigger = (electrics.values['stolAirBrakes'] or 0)
	local hoverTrigger = (electrics.values['stolHoverOn'] or 0)
	local parkBrakeTrigger = (electrics.values['brake_combined'] or 0)
	
	-- Detect air brake edge
	if airBrakeTrigger == 1 then	if lastTrigAirBrake == 0 then edgeDirAB = 1 end
	else							if lastTrigAirBrake == 1 then edgeDirAB = 2 end end
	
	-- Detect hover edge
	if hoverTrigger == 1 then	if lastTrigHover == 0 then edgeDirH = 1 end
	else						if lastTrigHover == 1 then edgeDirH = 2 end end
	
	-- Detect parking brake edge
	if parkBrakeTrigger == 1 then	if lastParkBrake == 0 then edgeDirPB = 1 end
	else							if lastParkBrake == 1 then edgeDirPB = 2 end end
	
	-- New trigger states have become old trigger states
	lastTrigAirBrake = airBrakeTrigger
	lastTrigHover = hoverTrigger
	lastParkBrake = parkBrakeTrigger
	
	-- Return result
	return edgeDirAB, edgeDirH, edgeDirPB
end



---------------------------------------------
-- Graphics update function                --
-- Updates other stuff too according to dt --
---------------------------------------------
local function updateGFX(dt)
	
	-- Detect trigger edges
	-- Retrieve some electrics
	local edgeDirAB
	local edgeDirH
	local edgeDirPB
	local airBrakeON	= (electrics.values['airBrakeON']	or 0)
	local hoverON		= (electrics.values['hoverON']		or 0)
	edgeDirAB, edgeDirH, edgeDirPB = getTriggerEdges()
	
	---------------
	-- Air Brake --
	---------------
	
	-- Air brake is momentary
	-- Rising edge detected  --> Air brake ON
	-- Falling edge detected --> Air brake OFF
	if airBrakeToggle == 0 then
		if edgeDirAB == 1 then
			electrics.values['airBrakeON'] = 1
			print("NOT toggling")
		elseif edgeDirAB == 2 then
			electrics.values['airBrakeON'] = 0
			print("NOT toggling")
		end
		
	-- Air brake is toggled on rising edge
	elseif edgeDirAB == 1 then
		if airBrakeON == 0 then
			electrics.values['airBrakeON'] = 1
		else
			electrics.values['airBrakeON'] = 0
		end
	end
	if electrics.values['airBrakeON'] == 1 then
		electrics.values['abs'] = 1
	else
		electrics.values['abs'] = 0
	end
		
		
	-----------
	-- Hover --
	-----------
	
	-- Hover is momentary
	-- Rising edge detected  --> Hover ON
	-- Falling edge detected --> Hover OFF
	if hoverToggle == 0 then
		if edgeDirH == 1 then
			electrics.values['hoverON'] = 1
			electrics.light_flash_highbeams(true)
		elseif edgeDirH == 2 then
			electrics.values['hoverON'] = 0
			electrics.light_flash_highbeams(false)
		end
		
	-- Hover is toggled on rising edge
	elseif edgeDirH == 1 then
		if hoverON == 0 then
			electrics.values['hoverON'] = 1
			electrics.light_flash_highbeams(true)
		else
			electrics.values['hoverON'] = 0
			electrics.light_flash_highbeams(false)
		end
	end
	
	------------------------------------
	-- BONUS: Parking brake indicator --
	------------------------------------
	if edgeDirPB == 1 then
		input.event("parkingbrake", 1, 0)
	elseif edgeDirPB == 2 then
		input.event("parkingbrake", 0, 0)
	end
	
	-- Debug message
	if ugfxCalled == 0 then
		ugfxCalled = 1
		print("ME262 STOL updateGFX called")
	end
end

-- Reset what we need to
local function reset()
	lastTrigAirBrake = 0
	lastTrigHover = 0
	electrics.values['airBrakeON'] = 0
	electrics.values['hoverON'] = 0
	print("ME262 STOL reset called")
end

-- Initialize everything
local function init(jbeamdata)
	hoverToggle = (v.data.variables["$hb9k_hoverToggle"].val or 1)
	airBrakeToggle = (v.data.variables["$hb9k_brakeToggle"].val or 0)
	print("ME262 STOL init called")
end

-- Public interface
M.init = init
M.reset = reset
M.updateGFX = updateGFX

-- Return the module
return M
